"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require('path');
const TransmuteFramework_1 = require("../../TransmuteFramework");
const accessControlArtifacts = require('../../../build/contracts/RBAC');
const eventStoreArtifacts = require('../../../build/contracts/RBACEventStore');
const eventStoreFactoryArtifacts = require('../../../build/contracts/RBACEventStoreFactory');
const { getCachedReadModel } = TransmuteFramework_1.default.EventStore;
const { web3, EventStoreContract, EventStoreFactoryContract, EventStore, TransmuteIpfs } = TransmuteFramework_1.default.init({
    env: 'testrpc',
    aca: accessControlArtifacts,
    esa: eventStoreArtifacts,
    esfa: eventStoreFactoryArtifacts,
});
exports.default = (vorpal) => {
    vorpal
        .command('eventstore create', 'Create an EventStore')
        .action((args, callback) => {
        web3.eth.getAccounts((err, addresses) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (err) {
                throw err;
            }
            if (args.options.from === undefined) {
                args.options.from = addresses[0];
            }
            let fromAddress = args.options.from;
            let factory = yield EventStoreFactoryContract.deployed();
            const { tx, events } = yield TransmuteFramework_1.default.Factory.createEventStore(factory, fromAddress);
            const contractAddress = events[0].payload.address;
            console.log('🎁  ' + contractAddress + ' EventStore created...');
            callback();
        }));
    });
    vorpal
        .command('eventstore permissions', 'Show an EventStore Permisssions')
        .option('-f, --from <from>', 'from address')
        .option('-c, --contractAddress <contractAddress>', 'contractAddress...')
        .types({
        string: ['contractAddress']
    })
        .action((args, callback) => {
        web3.eth.getAccounts((err, addresses) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (err) {
                throw err;
            }
            if (args.options.from === undefined) {
                args.options.from = addresses[0];
            }
            let fromAddress = args.options.from;
            let bindingModel = {
                contractAddress: args.options.contractAddress,
                fromAddress: args.options.from
            };
            let acc = yield EventStoreContract.at(bindingModel.contractAddress);
            let readModel = yield TransmuteFramework_1.default.Permissions.getPermissionsReadModel(acc, bindingModel.fromAddress);
            console.log(readModel);
            callback();
        }));
    });
    vorpal
        .command('eventstore grant', 'Create a permissions grant')
        .option('-c, --contractAddress <contractAddress>', 'contractAddress...')
        .option('-g, --grant <grant>', 'grant...')
        .types({
        string: ['contractAddress']
    })
        .action((args, callback) => {
        web3.eth.getAccounts((err, addresses) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (err) {
                throw err;
            }
            if (args.options.from === undefined) {
                args.options.from = addresses[0];
            }
            let grant = args.options.grant.split(",");
            let bindingModel = {
                contractAddress: args.options.contractAddress,
                fromAddress: args.options.from
            };
            let acc = yield EventStoreContract.at(bindingModel.contractAddress);
            let role = grant[0];
            let resource = grant[1];
            let action = grant[2];
            let attributes = eval(grant[3]);
            let { tx, events } = yield TransmuteFramework_1.default.Permissions.setGrant(acc, bindingModel.fromAddress, role, resource, action, attributes);
            let readModel = yield TransmuteFramework_1.default.Permissions.getPermissionsReadModel(acc, bindingModel.fromAddress);
            console.log(readModel);
            callback();
        }));
    });
    vorpal
        .command('eventstore can', 'Can role action resource')
        .option('-c, --contractAddress <contractAddress>', 'contractAddress...')
        .option('-q, --query <query>', 'query...')
        .types({
        string: ['contractAddress']
    })
        .action((args, callback) => {
        web3.eth.getAccounts((err, addresses) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (err) {
                throw err;
            }
            if (args.options.from === undefined) {
                args.options.from = addresses[0];
            }
            let query = args.options.query.split(",");
            let bindingModel = {
                contractAddress: args.options.contractAddress,
                fromAddress: args.options.from
            };
            let acc = yield EventStoreContract.at(bindingModel.contractAddress);
            let role = query[0];
            let action = query[1];
            let resource = query[2];
            let granted = yield TransmuteFramework_1.default.Permissions.canRoleActionResource(acc, bindingModel.fromAddress, role, action, resource);
            console.log(granted ? 'yes ' : 'no ', role, ' can ', action, resource);
            callback();
        }));
    });
    return vorpal;
};
