"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const { keys, pick, omit, flatten, difference, extend } = require('lodash');
let DEBUG = true;
const Common_1 = require("./Utils/Common");
const Common = require("./Utils/Common");
class EventStore {
    constructor(framework) {
        this.framework = framework;
        this.Common = Common;
        this.writeUnmarshalledEsCommand = (eventStore, fromAddress, esEvent) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let marshalledEvent = Common_1.marshal(esEvent.eventType, esEvent.keyType, esEvent.valueType, esEvent.key, esEvent.value);
            return yield eventStore.writeEvent(marshalledEvent.eventType, marshalledEvent.keyType, marshalledEvent.valueType, marshalledEvent.key, marshalledEvent.value, { from: fromAddress, gas: 4000000 });
        });
        this.readFSA = (eventStore, fromAddress, eventId) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let esEventValues = yield this.readEsEventValues(eventStore, fromAddress, eventId);
            let fsa = Common_1.getFSAFromEventValues(esEventValues[0], esEventValues[1], esEventValues[2], esEventValues[3], esEventValues[4], esEventValues[5], esEventValues[6], esEventValues[7]);
            if (fsa.meta.valueType === 'I') {
                if (!this.framework.TransmuteIpfs.ipfs) {
                    this.framework.TransmuteIpfs.init({
                        host: 'localhost',
                        port: '5001',
                        options: {
                            protocol: 'http'
                        }
                    });
                }
                let path = fsa.payload.multihash;
                fsa.payload = yield this.framework.TransmuteIpfs.readObject(path);
                fsa.payload = JSON.parse(JSON.stringify(fsa.payload));
                fsa.meta.multihash = path;
            }
            return fsa;
        });
        this.readEsEventValues = (eventStore, fromAddress, eventId) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield eventStore.readEvent.call(eventId, {
                from: fromAddress
            });
        });
        this.writeFSA = (eventStore, fromAddress, fsa) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (fsa.payload.length) {
                throw ('fsa payload cannot be an array');
            }
            let payloadKeys = Object.keys(fsa.payload);
            let valueType, keyType, key, value;
            if (payloadKeys.length > 1) {
                valueType = 'I';
                let hash = yield this.framework.TransmuteIpfs.writeObject(fsa.payload);
                key = 'multihash';
                value = hash;
            }
            else {
                valueType = Common.guessTypeFromValue(fsa.payload[payloadKeys[0]]);
                key = payloadKeys[0];
                value = fsa.payload[payloadKeys[0]];
            }
            let isHex = h => h.replace(/^0x/i, '').match(/[0-9A-Fa-f]+$/)
                ? h.replace(/^0x/i, '').match(/[0-9A-Fa-f]+$/)['index'] == 0
                : false;
            let formatHex = h => '0x' + h.replace(/^0x/i, '');
            if (key == 'bytes32') {
                if (valueType == 'X') {
                    if (value.length > 66)
                        throw ('solidity bytes32 type exceeded 32 bytes: ' + value.length + ' nybbles');
                    if (!isHex(value))
                        throw ('solidity bytes32 received invalid hex string: ' + value);
                }
                else if (valueType == 'S') {
                    if (value.length > 32)
                        throw ('solidity bytes32 type exceeded 32 bytes: ' + value.length + ' chars');
                }
                else
                    throw ('Invalid value type - ' + valueType);
            }
            else if (key == 'address') {
                if (valueType == 'A') {
                    if (!isHex(value))
                        throw ('solidity address received invalid hex string: ' + value);
                    if (formatHex(value).length != 42)
                        throw ('solidity address received invalid length: ' + value);
                }
                else
                    throw ('Invalid value type - ' + valueType);
            }
            let unmarshalledEsCommand = {
                eventType: fsa.type,
                keyType: 'X',
                valueType: valueType,
                key: key,
                value: value
            };
            let tx = yield this.writeUnmarshalledEsCommand(eventStore, fromAddress, unmarshalledEsCommand);
            let event = tx.logs[0].args;
            return Common.getFSAFromEventArgs(event);
        });
        this.readFSAs = (eventStore, fromAddress, eventId = 0) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let currentEvent = (yield eventStore.eventCount.call({ from: fromAddress })).toNumber();
            let eventPromises = [];
            while (eventId < currentEvent) {
                let fsa = yield this.readFSA(eventStore, fromAddress, eventId);
                eventPromises.push(fsa);
                eventId++;
            }
            return yield Promise.all(eventPromises);
        });
        this.writeFSAs = (eventStore, fromAddress, transmuteCommands) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let promises = transmuteCommands.map((cmd) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                return yield this.writeFSA(eventStore, fromAddress, cmd);
            }));
            return yield Promise.all(promises);
        });
    }
}
exports.EventStore = EventStore;
