"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const _ = require("lodash");
const Common = require("../Utils/Common");
const Reducer_1 = require("./Reducer");
class Permissions {
    constructor(framework) {
        this.framework = framework;
        this.setAddressRole = (acc, fromAddress, targetAddress, targetRole) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let tx = yield acc.setAddressRole(targetAddress, targetRole, {
                from: fromAddress,
                gas: 4000000
            });
            let fsa = Common.getFSAFromEventArgs(tx.logs[0].args);
            return {
                events: [fsa],
                tx: tx
            };
        });
        this.setGrant = (acc, fromAddress, role, resource, action, attributes) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let tx = yield acc.setGrant(role, resource, action, attributes, {
                from: fromAddress,
                gas: 4000000
            });
            let fsa = Common.getFSAFromEventArgs(tx.logs[1].args);
            return {
                events: [fsa],
                tx: tx
            };
        });
        this.getGrant = (acc, fromAddress, index) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let grantVals = yield acc.getGrant.call(index, {
                from: fromAddress
            });
            let grant = Common.grantItemFromValues(grantVals);
            return grant;
        });
        this.canRoleActionResource = (acc, fromAddress, role, action, resource) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let vals = yield acc.canRoleActionResource.call(role, action, resource, {
                from: fromAddress
            });
            return vals[0];
        });
    }
    getPermissionsReadModel(acc, fromAddress) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let events = yield this.framework.EventStore.readFSAs(acc, fromAddress, 0);
            events = _.filter(events, (event) => {
                return event.type === 'AC_GRANT_WRITTEN';
            });
            events = events.map((event) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                event.payload.grant = yield this.getGrant(acc, fromAddress, event.payload.index);
                event.meta.extended = ['grant'];
                return event;
            }));
            events = yield Promise.all(events);
            let readModel = this.framework.ReadModel.readModelGenerator(Reducer_1.readModel, Reducer_1.reducer, events);
            readModel.contractAddress = acc.address;
            readModel.readModelStoreKey = `${readModel.readModelType}:${readModel.contractAddress}`;
            return readModel;
        });
    }
}
exports.Permissions = Permissions;
