"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const LocalStorage_1 = require("./LocalStore/LocalStorage");
const moment = require('moment');
const DEBUG = false;
const log = DEBUG ? console.log : () => { };
var Persistence;
(function (Persistence) {
    Persistence.LocalStore = LocalStorage_1.LocalStorage;
    Persistence.FireStore = null;
    Persistence.expireItem = (key, store = Persistence.LocalStore) => {
        return new Promise((resolve, reject) => {
            let cacheObject = {
                expires: moment().subtract(1, "second").toISOString(),
                value: {}
            };
            store.setItem(key, cacheObject)
                .then((data) => {
                log('cache: ' + key + ' expired.');
                resolve(data.value);
            });
        });
    };
    Persistence.setItem = (key, value, expires = moment().add(15, 'seconds').toISOString(), store = Persistence.LocalStore) => {
        return new Promise((resolve, reject) => {
            let cacheObject = {
                expires: expires,
                value: value
            };
            store.setItem(key, cacheObject)
                .then((data) => {
                log('cache: ' + key + ' set.');
                resolve(data.value);
            });
        });
    };
    Persistence.getItem = (key, store = Persistence.LocalStore) => {
        return new Promise((resolve, reject) => {
            store.getItem(key)
                .then((data) => {
                if (moment().isAfter(moment(data.expires))) {
                    log('cache: ' + key + ' has expired.');
                    resolve(null);
                }
                else {
                    log('cache: ' + key + ' hit.');
                    resolve(data.value);
                }
            });
        });
    };
})(Persistence = exports.Persistence || (exports.Persistence = {}));
