"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EventStore_1 = require("./EventStore/EventStore");
const ReadModel_1 = require("./EventStore/ReadModel/ReadModel");
const Factory_1 = require("./EventStore/Factory/Factory");
const Persistence_1 = require("./EventStore/Persistence/Persistence");
const PatchLogic_1 = require("./EventStore/ReadModel/PatchLogic/PatchLogic");
const Permissions_1 = require("./EventStore/Permissions/Permissions");
const TransmuteIpfs_1 = require("./TransmuteIpfs/TransmuteIpfs");
const Toolbox_1 = require("./Toolbox/Toolbox");
const Web3 = require('web3');
const contract = require('truffle-contract');
let web3;
const accessControlArtifacts = require('../build/contracts/RBAC');
const eventStoreArtifacts = require('../build/contracts/RBACEventStore');
const eventStoreFactoryArtifacts = require('../build/contracts/RBACEventStoreFactory');
var ProviderEngine = require("web3-provider-engine");
var WalletSubprovider = require('web3-provider-engine/subproviders/wallet.js');
var Web3Subprovider = require("web3-provider-engine/subproviders/web3.js");
const config = {
    providerUrl: 'http://localhost:8545',
    aca: accessControlArtifacts,
    esa: eventStoreArtifacts,
    esfa: eventStoreFactoryArtifacts,
};
class TransmuteFramework {
    constructor() {
        this.AccessControlContract = null;
        this.EventStoreFactoryContract = null;
        this.EventStoreContract = null;
        this.EventStore = null;
        this.config = config;
        this.web3 = null;
        this.engine = null;
        this.TransmuteIpfs = TransmuteIpfs_1.TransmuteIpfs;
        this.Persistence = Persistence_1.Persistence;
        this.Toolbox = null;
        this.initWeb3 = (providerUrl) => {
            this.engine = new ProviderEngine();
            if (this.config.wallet) {
                this.engine.addProvider(new WalletSubprovider(this.config.wallet, {}));
            }
            this.engine.addProvider(new Web3Subprovider(new Web3.providers.HttpProvider(providerUrl)));
            this.engine.start();
            var web3 = new Web3(this.engine);
            this.web3 = web3;
        };
        this.init = (confObj = config) => {
            this.config = confObj;
            this.initWeb3(this.config.providerUrl);
            try {
                if (window.web3) {
                    console.info('Transmute Framework has detected MetaMask!');
                    this.web3 = window.web3;
                }
            }
            catch (e) {
            }
            if (this.web3) {
                this.AccessControlContract = contract(this.config.aca);
                this.AccessControlContract.setProvider(this.web3.currentProvider);
                this.EventStoreFactoryContract = contract(this.config.esfa);
                this.EventStoreFactoryContract.setProvider(this.web3.currentProvider);
                this.EventStoreContract = contract(this.config.esa);
                this.EventStoreContract.setProvider(this.web3.currentProvider);
            }
            else {
                console.warn('web3 is not available, install metamask.');
            }
            this.initAll();
            return this;
        };
        this.initAll = () => {
            let ipfsConfig = this.config.ipfsConfig || {
                host: 'localhost',
                port: '5001',
                options: {
                    protocol: 'http'
                }
            };
            this.TransmuteIpfs = TransmuteIpfs_1.TransmuteIpfs.init(ipfsConfig);
            this.Factory = new Factory_1.Factory(this);
            this.EventStore = new EventStore_1.EventStore(this);
            this.ReadModel = new ReadModel_1.ReadModel(this);
            this.PatchLogic = new PatchLogic_1.PatchLogic(this);
            this.Permissions = new Permissions_1.Permissions(this);
            this.Toolbox = new Toolbox_1.Toolbox(this);
        };
        this.getAccounts = () => {
            return new Promise((resolve, reject) => {
                this.web3.eth.getAccounts((err, addresses) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(addresses);
                    }
                });
            });
        };
    }
}
exports.TransmuteFramework = TransmuteFramework;
exports.default = new TransmuteFramework();
